@@ -502,7 +502,7 @@ class LopperSDT:
                 print( "[ERROR]: output file %s exists and force overwrite is not enabled" % output_filename )
                 sys.exit(1)
 
-            yaml = LopperYAML( None, self.tree, config=config )
+            yaml = LopperYAML( None, self.tree, config=self.config )
             yaml.to_yaml( output_filename )
         else:
             # we use the outfile extension as a mask
@@ -909,10 +909,18 @@ class LopperSDT:
                             prop_val = ""
 
                         if node_regex:
-                            if selected_nodes_possible:
-                                selected_nodes_possible = selected_nodes_possible + tree.nodes( node_regex )
+                            if node_regex.startswith( "/" ):
+                                if selected_nodes_possible:
+                                    selected_nodes_possible = selected_nodes_possible + tree.nodes( node_regex )
+                                else:
+                                    selected_nodes_possible = tree.nodes( node_regex )
                             else:
-                                selected_nodes_possible = tree.nodes( node_regex )
+                                # search with it as a label
+                                if selected_nodes_possible:
+                                    selected_nodes_possible = selected_nodes_possible + tree.lnodes( node_regex )
+                                else:
+                                    selected_nodes_possible = tree.lnodes( node_regex )
+
                         else:
                             # if the node_regex is empty, we operate on previously
                             # selected nodes.
@@ -1028,13 +1036,36 @@ class LopperSDT:
         if re.search( ".*,meta.*$", lop_type ):
             if re.search( "phandle-desc", lop_args ):
                 if self.verbose > 1:
-                    print( "[DBG++]: processing phandle meta data" )
-                Lopper.phandle_possible_prop_dict = OrderedDict()
+                    print( "[DBG++]: processing phandle meta data (%s)" % type(Lopper))
+
+                # grab all the defaults
+                Lopper.phandle_possible_prop_dict = Lopper.phandle_possible_properties()
+                try:
+                    del Lopper.phandle_possible_prop_dict["DEFAULT"]
+                except:
+                    pass
+
+                # now override, remove, extend
                 for p in lop_node:
                     # we skip compatible, since that is actually the compatibility value
                     # of the node, not a meta data entry. Everything else is though
                     if p.name != "compatible":
-                        Lopper.phandle_possible_prop_dict[p.name] = [ p.value[0] ]
+                        if re.search( r'^reset$', p.name ):
+                            if self.verbose > 1:
+                                print( "[DBG++]: resetting phandle table" )
+                            Lopper.phandle_possible_prop_dict = OrderedDict()
+                        elif re.search( r'^lopper-comment.*', p.name ):
+                            # skip
+                            pass
+                        elif re.search( r'^\-.*', p.name ):
+                            # delete
+                            p.name = re.sub( r'^\-', '', p.name )
+                            try:
+                                del Lopper.phandle_possible_prop_dict[p.name]
+                            except:
+                                pass
+                        else:
+                            Lopper.phandle_possible_prop_dict[p.name] = [ p.value[0] ]
 
         if re.search( ".*,output$", lop_type ):
             try:
@@ -1200,7 +1231,9 @@ class LopperSDT:
                 # select some nodes!
                 if "*" in tree_regex:
                     new_tree = LopperTree( True )
+                    new_tree.strict = False
                     new_tree.load( Lopper.export( self.FDT ) )
+                    new_tree.resolve()
                     new_tree.strict = not self.permissive
                 else:
                     # we can gather the tree nodes and unify with the selected
